@@ -3,7 +3,7 @@
 # portsreinstall-chroot library script
 # Overlay onto lib/libfs.sh for portsreinstall-chroot
 # - File system operations -
-# Copyright (C) 2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2018-2022 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -61,14 +61,18 @@ fs_save_mounttime_systembase ()
 # ============= Get the system base observed currently =============
 fs_get_current_systembase ()
 {
-	cat  "${TMPDIR}/fs_save_current_systembase" 2> /dev/null
+	local systembase
+	systembase=`cat "${TMPDIR}/fs_save_current_systembase" 2> /dev/null`
+	[ -z "$systembase" ] || realpath "$systembase"
 }
 
 # ============= Get the system base observed at the time of mounting =============
 # Non-zero return means no file system was attempted to mount
 fs_get_mounttime_systembase ()
 {
-	cat "${DBDIR}/fs_systembase" 2> /dev/null
+	local systembase
+	systembase=`cat "${DBDIR}/fs_systembase" 2> /dev/null`
+	[ -z "$systembase" ] || realpath "$systembase"
 }
 
 # ============= Get the system base in the scope of accessing =============
@@ -104,40 +108,89 @@ fs_build_chroot ()
 	fs_safeguard_basedir "$opt_basedir"
 	fs_unmount || return
 	mkdir -p "$opt_basedir"
-	# Prescan the f file system of the original environment
+	# Prescan the file system of the original environment
 	cp /dev/null "${TMPDIR}/fs_build_chroot:directories"
 	(
+		real_basedir=`realpath "$opt_basedir"`
 		{
+# 			echo bin compat etc lib libexec root sbin sys usr | tr ' ' '\n'
 			echo bin compat etc lib libexec root sbin sys usr var | tr ' ' '\n'
 			echo "$opt_extra_dirs" | tr "$opt_extra_dirs_delim" '\n'
-		} | grep -v '^[[:space:]]*$' | sort -u > ${TMPDIR}/fs_build_chroot:sys_dirs
-		sysdirs_ptn="^/*(`str_escape_regexp_filter < ${TMPDIR}/fs_build_chroot:sys_dirs | tr '\n' '|' | sed 's/\|$//'`)/+"
-		while read directory
+		} | env LANG=C grep -v '^[[:space:]]*$' | sort -u | while read node
 		do
-			[ -e "/$directory" ] || continue
-			if [ -L "/$directory" ]
+			[ -e "/$node" ] || continue
+			src_mountpoint_real=`realpath "/$node"`
+			ptn_src_mountpoint_real=`str_escape_regexp "$src_mountpoint_real/"`
+			if echo "$real_basedir" | grep -Eq "^$ptn_src_mountpoint_real"
 			then
-				src_mountpoint_real=`realpath "/$directory"`
-					printf '%s\t%s\n' link "$directory" >> ${TMPDIR}/fs_build_chroot:directories
-				if ! echo "$src_mountpoint_real/" | grep -qE "$sysdirs_ptn"
+				(
+					node_cur=$node
+					rm -f "${TMPDIR}/fs_build_chroot:hit_exact"
+					while [ ! -e "${TMPDIR}/fs_build_chroot:hit_exact" ]
+					do
+						rm -f "${TMPDIR}/fs_build_chroot:hit_subnode"
+						cd "/$node_cur"
+						ls -a | while read subnode
+						do
+							if [ "$subnode" = . -o "$subnode" = .. ]
+							then
+								continue
+							elif [ -L "$subnode" -o -f "$subnode" ]
+							then
+								echo "$node_cur/$subnode"
+							elif [ -d "$subnode" ]
+							then
+								node_cur_tmp_real=`realpath "/$node_cur/$subnode"`
+								ptn_node_cur_tmp_real=`str_escape_regexp "$node_cur_tmp_real/"`
+								if [ "$real_basedir" = "$node_cur_tmp_real" ]
+								then
+									touch "${TMPDIR}/fs_build_chroot:hit_exact"
+								elif echo "$real_basedir" | grep -Eq "^$ptn_node_cur_tmp_real"
+								then
+									echo "$subnode" > ${TMPDIR}/fs_build_chroot:hit_subnode
+								else
+									echo "$node_cur/$subnode"
+								fi
+							fi
+						done
+						[ -e "${TMPDIR}/fs_build_chroot:hit_subnode" ] || break
+						node_cur=$node_cur/`cat "${TMPDIR}/fs_build_chroot:hit_subnode"`
+					done
+				)
+			else
+				echo "$node"
+			fi
+		done > ${TMPDIR}/fs_build_chroot:sys_nodes
+		sysdirs_ptn="^/*(`str_escape_regexp_filter < ${TMPDIR}/fs_build_chroot:sys_nodes | tr '\n' '|' | sed 's/\|$//'`)/+"
+		while read node
+		do
+			[ -e "/$node" ] || continue
+			if [ -L "/$node" -a -d "/$node" ]
+			then
+				src_mountpoint_real=`realpath "/$node"`
+					printf '%s\t%s\n' link "$node" >> ${TMPDIR}/fs_build_chroot:directories
+				if ! echo "$src_mountpoint_real/" | env LANG=C grep -qE "$sysdirs_ptn"
 				then
 					printf '%s\t%s\n' real "$src_mountpoint_real" >> ${TMPDIR}/fs_build_chroot:directories
 					tmpdir_descendant=${TMPDIR}/fs_build_chroot:descendant/$src_mountpoint_real
 					mkdir -p "$tmpdir_descendant"
 					fs_get_descendant_mount_info "/$src_mountpoint_real" > $tmpdir_descendant/list
 				fi
-			elif [ -d "/$directory" ]
+			elif [ -d "/$node" ]
 			then
-				printf '%s\t%s\n' real $directory >> ${TMPDIR}/fs_build_chroot:directories
-				tmpdir_descendant=${TMPDIR}/fs_build_chroot:descendant/$directory
+				printf '%s\t%s\n' real $node >> ${TMPDIR}/fs_build_chroot:directories
+				tmpdir_descendant=${TMPDIR}/fs_build_chroot:descendant/$node
 				mkdir -p "$tmpdir_descendant"
-				fs_get_descendant_mount_info "/$directory" > $tmpdir_descendant/list
+				fs_get_descendant_mount_info "/$node" > $tmpdir_descendant/list
+			elif [ -L "/$node" -o -f "/$node" ]
+			then
+				cp -p "/$node" "$node" 
 			fi
-		done < ${TMPDIR}/fs_build_chroot:sys_dirs
+		done < ${TMPDIR}/fs_build_chroot:sys_nodes
 	)
 	# Prepare the grand base of the chroot environment
 	(
-		cd "/$opt_basedir"
+		cd "$opt_basedir"
 		for directory in builder mask store
 		do
 			[ -d $directory ] || mkdir $directory
@@ -149,14 +202,16 @@ fs_build_chroot ()
 		echo "$PORTSDIR"
 		echo "$PORTSNAP_WORKDIR"
 		echo "$PKGNG_PKG_CACHEDIR"
-	fi | str_regularize_df_path_filter | grep -v '^[[:space:]]*$' | sort -u > ${DBDIR}/shared_dirs.lst
+	fi | str_regularize_df_path_filter | env LANG=C grep -v '^[[:space:]]*$' | sort -u > ${DBDIR}/shared_dirs.lst
 	str_escape_regexp_filter < ${DBDIR}/shared_dirs.lst | sed 's|^|^|;s|$|\/|' > ${TMPDIR}/fs_build_chroot:shared_dirs.regexp.tmp
 	paste "${DBDIR}/shared_dirs.lst" "${TMPDIR}/fs_build_chroot:shared_dirs.regexp.tmp" > ${TMPDIR}/fs_build_chroot:shared_dirs.regexp
 	cp /dev/null "${TMPDIR}/fs_build_chroot:shared_dirs:added"
 	# Build target directories and the manifest for mounting
+	mkdir -p "$opt_basedir/mask"
 	cp /dev/null "${DBDIR}/mount_manifest.tmp"
 	(
-		cd "/$opt_basedir"/builder
+		real_basedir=`realpath "$opt_basedir"`
+		cd "$real_basedir"/builder
 		while read srcline
 		do
 			type=`echo "$srcline" | cut -f 1`
@@ -167,7 +222,7 @@ fs_build_chroot ()
 				;;
 			real )
 				mkdir -p "./$directory"
-				masktarget=/$opt_basedir/mask/$directory
+				masktarget=$real_basedir/mask/$directory
 				mkdir -p "$masktarget"
 				printf '%s\t%s\t%s\t%s\n' nullfs "/$directory" "$directory" ro >> ${DBDIR}/mount_manifest.tmp
 				printf '%s\t%s\t%s\t%s\n' unionfs "$masktarget" "$directory" rw,noatime >> ${DBDIR}/mount_manifest.tmp
@@ -181,10 +236,10 @@ fs_build_chroot ()
 					rm -f "${TMPDIR}/fs_build_chroot:shared_dirs:is_itself"
 					while read -r shared_path shared_path_regexp
 					do
-						echo "$fullpath/" | grep -qE "$shared_path_regexp" || continue
+						echo "$fullpath/" | env LANG=C grep -qE "$shared_path_regexp" || continue
 						echo "$shared_path"$'\n'"$fullpath" | while read mpath
 						do
-							if  ! grep -qFx "$mpath" "${TMPDIR}/fs_build_chroot:shared_dirs:added"
+							if  ! env LANG=C grep -qFx "$mpath" "${TMPDIR}/fs_build_chroot:shared_dirs:added"
 							then
 								echo "$mpath" >> ${TMPDIR}/fs_build_chroot:shared_dirs:added
 								mkdir -p "/$mpath"
@@ -197,7 +252,7 @@ fs_build_chroot ()
 					[ -e "${TMPDIR}/fs_build_chroot:shared_dirs:is_under" ] && continue
 					case $fs in
 						normal )
-							masktarget=`str_regularize_df_path "/$opt_basedir/mask/$fullpath"`
+							masktarget=`str_regularize_df_path "$real_basedir/mask/$fullpath"`
 							mkdir -p "$masktarget"
 							printf '%s\t%s\t%s\t%s\n' nullfs "$mp" "$fullpath" ro >> ${DBDIR}/mount_manifest.tmp
 							printf '%s\t%s\t%s\t%s\n' unionfs "$masktarget" "$fullpath" rw,noatime >> ${DBDIR}/mount_manifest.tmp
@@ -222,7 +277,7 @@ fs_build_chroot ()
 				;;
 			esac
 		done < ${TMPDIR}/fs_build_chroot:directories
-		grep -Ev -f "${TMPDIR}/fs_build_chroot:shared_dirs:added" "${DBDIR}/shared_dirs.lst" | while read shared_dir
+		env LANG=C grep -Ev -f "${TMPDIR}/fs_build_chroot:shared_dirs:added" "${DBDIR}/shared_dirs.lst" | while read shared_dir
 		do
 			mkdir -p "$shared_dir"
 			mp_share=`realpath "$shared_dir"`
@@ -237,7 +292,7 @@ fs_build_chroot ()
 		printf '%s\t%s\t%s\t%s\n' procfs proc proc rw >> ${DBDIR}/mount_manifest.tmp
 		printf '%s\t%s\t%s\t%s\n' tmpfs tmpfs tmp rw,mode=1777 >> ${DBDIR}/mount_manifest.tmp
 		mkdir -p ."${PROGRAM}"
-		cd "/$opt_basedir/mask"
+		cd "$real_basedir/mask"
 		if [ ! -e root/.cshrc ]
 		then
 			tmp_cshrc=${TMPDIR}/fs_build_chroot:.cshrc
@@ -257,7 +312,7 @@ fs_build_chroot ()
 			echo 'set prompt="%N@[builder]%m:%~ %# "' >> $tmp_cshrc
 			mv "$tmp_cshrc" root/.cshrc
 		fi
-		printf '%s\t%s\t%s\t%s\n' nullfs "/$opt_basedir"/store ".${PROGRAM}" rw >> ${DBDIR}/mount_manifest.tmp
+		printf '%s\t%s\t%s\t%s\n' nullfs "$real_basedir"/store ".${PROGRAM}" rw >> ${DBDIR}/mount_manifest.tmp
 	)
 	mv "${DBDIR}/mount_manifest.tmp" "${DBDIR}/mount_manifest"
 }
@@ -279,7 +334,8 @@ fs_chk_mount ()
 		directory=`echo "$srcline" | cut -f 3`
 		opt=`echo "$srcline" | cut -f 4`
 		mp=`str_regularize_df_path "$systembase_mp/$opt_basedir/builder/$directory"`
-		if ! fs_chk_mounted "$type" "$target" "$mp"
+		expr "$target" : \\/ && target=`realpath "$target"`
+		if ! real_mp=`realpath "$mp" 2> /dev/null` || ! fs_chk_mounted "$type" "$target" "$real_mp"
 		then
 			printf '%s\t%s\t%s\n' "$type" "$target" "$mp" >> $tmp_remains
 		fi
@@ -412,8 +468,9 @@ fs_chk_unmount ()
 		[ "x$type" = xnullfs -o "x$type" = xunionfs ] && target=$systembase_target/$target
 		directory=`echo "$srcline" | cut -f 3`
 		opt=`echo "$srcline" | cut -f 4`
-		mp=$systembase_mp/$opt_basedir/builder/$directory
-		fs_chk_mounted "$type" "$target" "$mp" || continue
+		mp=`str_regularize_df_path "$systembase_mp/$opt_basedir/builder/$directory"`
+		real_mp=`realpath "$mp" 2> /dev/null` || continue
+		fs_chk_mounted "$type" "$target" "$real_mp" || continue
 		str_regularize_df_path "$mp" >> $tmp_remains
 	done
 	! cat "$tmp_remains" 2> /dev/null
