@@ -3,13 +3,13 @@
 # portsreinstall library script
 # Overlay onto lib/liboptions.sh for portsreinstall-chroot
 # - Command line options -
-# Copyright (C) 2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2018-2022 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
 # ============= Database of options which are given at each run and not saved =============
 # [Syntax of option databases]
-# short_name, long_name, variable, defult_vaule, set_value
+# short_name, long_name, variable, default_value, set_value
 # Columns are delimited by tab characters.
 options_db_onetime ()
 {
@@ -43,8 +43,16 @@ s	share-port-pkgs-dirs	opt_share_port_pkgs_dirs	no	yes
 eof
 }
 
-# ============= Regularize the option value =============
-options_regularize ()
+# ============= Definitions of value regulator functions =============
+options_db_value_regulators ()
 {
-	opt_basedir=`fs_global_path "$opt_basedir"`
+	cat << eof
+opt_basedir	options_regularize_to_global_path
+eof
+}
+
+# ============= Regularize a value of an option value: conversion of a oath into the global path =============
+options_regularize_to_global_path ()
+{
+	fs_global_path "$1"
 }
