@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - "do"/"redo" command operation -
-# Copyright (C) 2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2018-2022 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -224,7 +224,7 @@ command_do_reset_changed_portdb ()
 {
 	local tmp_ls
 	program_chk_stage_complete INSPECT_DEPENDENTS || return
-	pkgsys_exists_saved_port_oprions_timestamps || return
+	pkgsys_exists_saved_port_options_timestamps || return
 	message_section_title "Inspecting changes of port options"
 	tmp_ls=${TMPDIR}/command_do_reset_changed_portdb
 	rm -f "$tmp_ls.updated"
@@ -232,9 +232,9 @@ command_do_reset_changed_portdb ()
 	then
 		message_echo "WARNING: ${PORT_DBDIR} is missing. All port options are recognized as default."
 	fi
-	pkgsys_get_saved_port_oprions_timestamps_all > $tmp_ls.db
-	pkgsys_get_current_port_oprions_timestamp > $tmp_ls.log
-	pkgsys_get_changed_port_oprions "$tmp_ls.db" "$tmp_ls.log" > $tmp_ls.diff
+	pkgsys_get_saved_port_options_timestamps_all > $tmp_ls.db
+	pkgsys_get_current_port_options_timestamp > $tmp_ls.log
+	pkgsys_get_changed_port_options "$tmp_ls.db" "$tmp_ls.log" > $tmp_ls.diff
 	if ! [ `wc -l < $tmp_ls.diff` -gt 0 ]
 	then
 		message_echo
@@ -270,14 +270,14 @@ command_do_reset_changed_portdb ()
 		fi
 	done
 	message_echo "INFO: Recording for not inspected ports..."
-	pkgsys_conv_portorigin_to_port_oprion_timestamp_logname < $tmp_ls.uninspected.origin > $tmp_ls.uninspected.logname
+	pkgsys_conv_portorigin_to_port_option_timestamp_logname < $tmp_ls.uninspected.origin > $tmp_ls.uninspected.logname
 	str_escape_regexp_filter < $tmp_ls.uninspected.dbfile | sed 's/^/^/;s/$/[[:space:]]/' > $tmp_ls.uninspected.dbfile_ptn
 	paste "$tmp_ls.uninspected.dbfile_ptn" "$tmp_ls.uninspected.logname" | while read -r dbfile_ptn logname
 	do
-		grep -E "$dbfile_ptn" < $tmp_ls.log > ${DBDIR}/ls_dbdir/$logname.log 2> /dev/null || :
+		env LANG=C grep -E "$dbfile_ptn" < $tmp_ls.log > ${DBDIR}/ls_dbdir/$logname.log 2> /dev/null || :
 	done
 	str_escape_regexp_filter < $tmp_ls.nonexistent.db | sed 's/^/^/;s/$/[[:space:]]/' > $tmp_ls.nonexistent.db_ptn
-	grep -E -f "$tmp_ls.nonexistent.db_ptn" < $tmp_ls.log | pkgsys_register_list_nonexistent_portopriondb
+	env LANG=C grep -E -f "$tmp_ls.nonexistent.db_ptn" < $tmp_ls.log | pkgsys_register_list_nonexistent_portopriondb
 	message_echo
 	[ ! -e "$tmp_ls.updated" ]
 }
@@ -329,7 +329,7 @@ command_do_convert_dependency_lists_to_actual_ones ()
 						if [ -e "$target.src" ]
 						then
 							sed -E -f "${DBDIR}/REPLACE.complete_sed_pattern" "$target.src" \
-								| grep -v '^$' | sort -u > $target.tmp
+								| env LANG=C grep -v '^$' | sort -u > $target.tmp
 							[ -e "$target" ] && ! diff -q "$target.tmp" "$target" > /dev/null \
 								&& echo "$origin" >> ${DBDIR}/update_dependencies
 							mv "$target.tmp" "$target"
@@ -354,13 +354,13 @@ command_do_convert_dependency_lists_to_actual_ones ()
 				find . -depth 3 -type f -name requirements.${tag}.full -or -name requirements.${tag}.full.orig ) \
 				| sort -u \
 				| sed 's|^./||;s|/[^/]*$||' \
-				| grep -v -Fx -f "${DBDIR}/update_dependencies" \
+				| env LANG=C grep -v -Fx -f "${DBDIR}/update_dependencies" \
 				> ${TMPDIR}/convert_requirements_list:full_complete.grep_pattern || :
 			( cd "${DBDIR}/requires" && \
 				find . -depth 3 -type f -name requirements.${tag}.direct -or -name requirements.${tag}.direct.orig ) \
 				| sort -u \
 				| sed 's|^./||;s|/[^/]*$||' \
-				| grep -v -Fx -f "${TMPDIR}/convert_requirements_list:full_complete.grep_pattern" \
+				| env LANG=C grep -v -Fx -f "${TMPDIR}/convert_requirements_list:full_complete.grep_pattern" \
 				> ${DBDIR}/stage.loop_list/complete_recursive_${tag}time_reqlists || :
 		done
 		for inspected_level_tmp in direct node
@@ -423,7 +423,7 @@ command_do_trim_dependency_lists_by_removing_uninspected_ports ()
 				do
 					srcdb=requirements.${tag}.${level}
 					[ -e "$dbpath/$srcdb" ] || continue
-					grep -Fx -f "${DBDIR}/inspected_ports" "$dbpath/$srcdb" > $dbpath/$srcdb.tmp || :
+					env LANG=C grep -Fx -f "${DBDIR}/inspected_ports" "$dbpath/$srcdb" > $dbpath/$srcdb.tmp || :
 					mv "$dbpath/$srcdb.tmp" "$dbpath/$srcdb"
 				done
 			done
@@ -621,7 +621,7 @@ command_do_post_process_after_the_preparation_of_target_attribute_information ()
 	{
 		message_section_title "Post-process after the preparation of target attribute information"
 		sort -u "${DBDIR}/all_targets.lst" 2> /dev/null \
-			| grep -Fx -f "${DBDIR}/inspected_ports" \
+			| env LANG=C grep -Fx -f "${DBDIR}/inspected_ports" \
 			| sed -E -f "${DBDIR}/REPLACE.complete_sed_pattern" 2> /dev/null \
 			> ${DBDIR}/all_targets.lst.tmp || :
 		mv "${DBDIR}/all_targets.lst.tmp" "${DBDIR}/all_targets.lst"
@@ -792,7 +792,7 @@ command_do_preparation_for_inspection_of_new_leaf_ports ()
 			fileedit_manipulate_new_lines \
 				"${TMPDIR}/PREPARE_INSPECT_LEAF_PORTS:nonleaf_ports" \
 				"${TMPDIR}/PREPARE_INSPECT_LEAF_PORTS:inspected_ports" \
-				| grep -v -Fx -f "${DBDIR}/conf/HOLD:PORTS.parsed" 2> /dev/null \
+				| env LANG=C grep -v -Fx -f "${DBDIR}/conf/HOLD:PORTS.parsed" 2> /dev/null \
 				> ${DBDIR}/stage.loop_list/leaf_ports_primary_candidates || :
 			cp /dev/null "${DBDIR}/leaf_ports.filter"
 			cp /dev/null "${DBDIR}/stage.loop_list/leaf_ports_secondary_candidates"
@@ -815,8 +815,8 @@ command_do_inspection_of_new_primary_leaf_ports ()
 			origin=$1
 			pkgsys_is_pkgtool "$origin" && return
 			dbpath=${DBDIR}/requires/$origin
-			grep -q -Fx "$origin" "${DBDIR}/need.with_replaced.list" 2> /dev/null && return
-			if ! grep -q -Fx "$origin" "${DBDIR}/noneed.list" 2> /dev/null
+			env LANG=C grep -q -Fx "$origin" "${DBDIR}/need.with_replaced.list" 2> /dev/null && return
+			if ! env LANG=C grep -q -Fx "$origin" "${DBDIR}/noneed.list" 2> /dev/null
 			then
 				if [ -e "$dbpath/initial_orig" ]
 				then
@@ -832,7 +832,7 @@ command_do_inspection_of_new_primary_leaf_ports ()
 			if [ -e "$dbpath/requirements.all.full" -o -e "$dbpath/ignored_requirements.all" ]
 			then
 				cat "$dbpath/requirements.all.full" "$dbpath/ignored_requirements.all" 2> /dev/null | \
-					grep -v -Fx -f "${DBDIR}/conf/HOLD:PORTS.parsed" 2> /dev/null| \
+					env LANG=C grep -v -Fx -f "${DBDIR}/conf/HOLD:PORTS.parsed" 2> /dev/null| \
 					fileedit_add_lines_if_new "${DBDIR}/stage.loop_list/leaf_ports_secondary_candidates" || :
 			fi
 			fileedit_add_a_line_if_new "$origin" "${DBDIR}/leaf_ports.filter"
@@ -871,9 +871,9 @@ command_do_inspection_of_requirements_of_new_leaf_ports ()
 					origin=$1
 					pkgsys_is_pkgtool "$origin" && return
 					dbpath=${DBDIR}/requires/$origin
-					grep -q -Fx "$origin" "${DBDIR}/need.with_replaced.list" 2> /dev/null && return
+					env LANG=C grep -q -Fx "$origin" "${DBDIR}/need.with_replaced.list" 2> /dev/null && return
 					cat "$dbpath/dependents.all.full" "$dbpath/ignored_dependents.all"  2> /dev/null \
-						| grep -Fxq -v -f "${DBDIR}/leaf_ports.filter" 2> /dev/null && return
+						| env LANG=C grep -Fxq -v -f "${DBDIR}/leaf_ports.filter" 2> /dev/null && return
 					cat "$dbpath/requirements.all.full" "$dbpath/ignored_requirements.all" 2> /dev/null \
 						>> ${DBDIR}/leaf_ports_secondary_candidates.new_requirements || :
 					fileedit_add_a_line_if_new "$origin" "${DBDIR}/leaf_ports.filter"
@@ -889,10 +889,10 @@ command_do_inspection_of_requirements_of_new_leaf_ports ()
 					break
 				fi
 				{
-					grep -Fx -v -f "${DBDIR}/leaf_ports.filter" \
+					env LANG=C grep -Fx -v -f "${DBDIR}/leaf_ports.filter" \
 						"${DBDIR}/stage.loop_list/leaf_ports_secondary_candidates" || :
 					cat "${DBDIR}/leaf_ports_secondary_candidates.new_requirements" || :
-				} 2> /dev/null | grep -v -Fx -f "${DBDIR}/conf/HOLD:PORTS.parsed" 2> /dev/null | sort -u \
+				} 2> /dev/null | env LANG=C grep -v -Fx -f "${DBDIR}/conf/HOLD:PORTS.parsed" 2> /dev/null | sort -u \
 					> ${DBDIR}/stage.loop_list/leaf_ports_secondary_candidates.tmp || :
 				program_reset_loop_for_stage INSPECT_REQUIREMENTS_OF_LEAF_PORTS
 				mv "${DBDIR}/stage.loop_list/leaf_ports_secondary_candidates.tmp" \
@@ -902,7 +902,7 @@ command_do_inspection_of_requirements_of_new_leaf_ports ()
 				num_inspect=`wc -l < ${DBDIR}/stage.loop_list/leaf_ports_secondary_candidates | tr -d ' '`
 				message_echo "  $num_leaves_new leaf port(s) is/are newly found; continue for $num_inspect candidate(s)."
 			done
-			grep -Fx -f "${DBDIR}/leaf_ports.filter" "${DBDIR}/inspected_ports" 2> /dev/null | sort -u > ${DBDIR}/leaf_ports || :
+			env LANG=C grep -Fx -f "${DBDIR}/leaf_ports.filter" "${DBDIR}/inspected_ports" 2> /dev/null | sort -u > ${DBDIR}/leaf_ports || :
 			message_echo
 		}
 		program_exec_and_record_completion INSPECT_REQUIREMENTS_OF_LEAF_PORTS
@@ -1008,7 +1008,7 @@ command_do_collection_of_obsolete_ports_to_delete ()
 			cat "${DBDIR}/initial/$origin/requirements.run.full" || :
 			cat "${DBDIR}/obsolete/$origin/requirements.run.full" || :
 		done | sort -u > $reqptn_file
-		grep -v -Fx -f "$reqptn_file" "$src_selected" > $dst_selected 2> /dev/null || :
+		env LANG=C grep -v -Fx -f "$reqptn_file" "$src_selected" > $dst_selected 2> /dev/null || :
 		message_echo
 	}
 	program_exec_and_record_completion COLLECT_OBSOLETE_PORTS_TO_DELETE
@@ -1049,7 +1049,7 @@ command_do_composition_of_a_list_for_deinstallation_of_obsolete_and_leaf_package
 		if [ ! -e "${DBDIR}/inspected_ports_only_partially" ]
 		then
 			cat "$reqptn_leaf" "$reqptn_obs" 2> /dev/null | sort -u > $grepptn
-			grep -v -Fx -f "$grepptn" "$leaf_selected_src" 2> /dev/null \
+			env LANG=C grep -v -Fx -f "$grepptn" "$leaf_selected_src" 2> /dev/null \
 				| database_query_add_initial_origins > $leaf_selected || :
 			cat "$obs_selected" "$leaf_selected" 2> /dev/null || :
 		else
@@ -1059,11 +1059,11 @@ command_do_composition_of_a_list_for_deinstallation_of_obsolete_and_leaf_package
 		str_escape_regexp_filter < ${DBDIR}/stage.loop_list/ports_to_delete \
 			| sed 's/^/^/;s/$/[[:space:]]/' > $grepptn_col1
 		cat "${DBDIR}/leaf_ports.with_ini" "${DBDIR}/obsolete_ports" 2> /dev/null \
-			| grep -Fx -v -f "${DBDIR}/stage.loop_list/ports_to_delete" > ${DBDIR}/stage.loop_list/ports_to_restore 2> /dev/null || :
+			| env LANG=C grep -Fx -v -f "${DBDIR}/stage.loop_list/ports_to_delete" > ${DBDIR}/stage.loop_list/ports_to_restore 2> /dev/null || :
 		if [ $opt_batch_mode = no ]
 		then
 			if [ ! -e "${DBDIR}/inspected_ports_only_partially" ] && \
-				grep -v -Fx -f "${DBDIR}/stage.loop_list/ports_to_delete" "$leaf_selected_src" > $preserved 2> /dev/null
+				env LANG=C grep -v -Fx -f "${DBDIR}/stage.loop_list/ports_to_delete" "$leaf_selected_src" > $preserved 2> /dev/null
 			then
 				message_echo "INFO: Following leaf ports are preserved because required by other preserved leaf/obsolete ports."
 				message_echo "----------------"
@@ -1079,7 +1079,7 @@ command_do_composition_of_a_list_for_deinstallation_of_obsolete_and_leaf_package
 				done < $preserved
 				message_echo "----------------"
 			fi
-			if grep -v -Fx -f "${DBDIR}/stage.loop_list/ports_to_delete" "$obs_selected_src" > $preserved 2> /dev/null
+			if env LANG=C grep -v -Fx -f "${DBDIR}/stage.loop_list/ports_to_delete" "$obs_selected_src" > $preserved 2> /dev/null
 			then
 				message_echo "INFO: Following obsolete ports are preserved because required by other obsolete ports."
 				message_echo "----------------"
@@ -1112,7 +1112,7 @@ command_do_collect_entire_distfiles_list ()
 		{
 			message_section_title "Collecting entire distfiles list"
 			find "${PORTSDIR}" -depth 3 -name distinfo -exec cat {} \; \
-				| grep '^SHA256 ' | sed -E 's/^SHA256 \(([^)]*)\).*/\1/' \
+				| env LANG=C grep '^SHA256 ' | sed -E 's/^SHA256 \(([^)]*)\).*/\1/' \
 				| sort -u > ${DBDIR}/distfiles.entire.tmp
 			mv "${DBDIR}/distfiles.entire.tmp" "${DBDIR}/distfiles.entire"
 			message_echo
@@ -1292,7 +1292,7 @@ command_do_rebuild_of_package_database ()
 	{
 		which -s pkgdb || return 0
 		message_section_title "Rebuilding package database for portupgrade"
-		if grep -q @ "${DBDIR}/stage.loop_list/reinst_todo"
+		if env LANG=C grep -q @ "${DBDIR}/stage.loop_list/reinst_todo"
 		then
 			message_echo "INFO: Skipped because of lacking flavor support."
 		else
