@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - Commands of flexible configuration -
-# Copyright (C) 2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2018-2022 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -100,8 +100,8 @@ command_flexconf_update_freeze ()
 	fileedit_combine_lists "${DBDIR}/conf/FREEZE:PORTS.parsed" "${DBDIR}/freeze.list" > ${DBDIR}/freeze.all.list
 	rm -f "$tmp_list:exists_update"
 	{
-		grep -vFx -f "$tmp_list:old" "${DBDIR}/freeze.all.list"
-		grep -vFx -f "${DBDIR}/freeze.all.list" "$tmp_list:old"
+		env LANG=C grep -vFx -f "$tmp_list:old" "${DBDIR}/freeze.all.list"
+		env LANG=C grep -vFx -f "${DBDIR}/freeze.all.list" "$tmp_list:old"
 	} | while read origin
 	do
 		touch "$tmp_list:exists_update"
