@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - "forget" command operation -
-# Copyright (C) 2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2018-2022 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -74,7 +74,7 @@ command_forget ()
 		origin=$1
 		origins_init=`database_query_initial_orgins "$origin"`
 		if [ -z "$origins_init" ] \
-			&& ! grep -qv -Fx -f "${DBDIR}/forget/remove.scope.filter" \
+			&& ! env LANG=C grep -qv -Fx -f "${DBDIR}/forget/remove.scope.filter" \
 				"${DBDIR}/requires/$origin/dependents.all.full" \
 				2> /dev/null
 		then
@@ -94,7 +94,7 @@ command_forget ()
 			cat "${DBDIR}/forget/remove" || :
 		} 2> /dev/null | sort -u > ${DBDIR}/forget/remove.filter
 		cat "${DBDIR}/inspected_ports.update" 2> /dev/null | sort -u \
-			| grep -v -Fx -f "${DBDIR}/forget/remove.filter" \
+			| env LANG=C grep -v -Fx -f "${DBDIR}/forget/remove.filter" \
 				> ${DBDIR}/inspected_ports.update.tmp 2> /dev/null || :
 		mv "${DBDIR}/inspected_ports.update.tmp" "${DBDIR}/inspected_ports.update"
 		message_echo
