@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - Commands of ports/packages operations -
-# Copyright (C) 2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2018-2022 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -77,7 +77,7 @@ command_pkgs_restore ()
 	do
 		pkgsys_register_evaluated_globs remove "${DBDIR}/taboo.list" "$origin"
 		message_echo "  Deregistered $origin from taboo."
-		grep -Fx -q "$origin" "$tmp_done_orig" 2> /dev/null || :
+		env LANG=C grep -Fx -q "$origin" "$tmp_done_orig" 2> /dev/null || :
 		if pkgsys_exists_from_orig "$origin"
 		then
 			pkg=`pkgsys_get_installed_pkg_from_origin "$origin"`
@@ -159,7 +159,7 @@ command_pkgs_pkgsanity ()
 		while read origin
 		do
 			pkgsys_get_installed_pkg_from_origin "$origin"
-		done < $tmp_list.orgs | grep -v '^[[:space:]]*$' > $tmp_list.pkgs || {
+		done < $tmp_list.orgs | env LANG=C grep -v '^[[:space:]]*$' > $tmp_list.pkgs || {
 			message_echo "WARNING: No such globs match any installed package." >&2
 			temp_terminate_process () { :; }
 			exit 1
@@ -210,32 +210,34 @@ command_pkgs_pkgsanity ()
 	do
 		pkg=`sed -n ${iline}p "$tmp_list.pkgs"`
 		iline=$((${iline}+1))
-		origin=`pkg_info_flavored_origin "$pkg"`
-		[ -n "$origin" ] || continue
-		grep -q -Fx "$origin" "${DBDIR}/damaged_package" 2>/dev/null && continue
-		pkgsys_sanitychk_pkgcontents "$pkg" is_reinstall_encouraged && continue
-		port_path=`pkgsys_get_portpath_from_origin "$origin"`
-		if [ ! -d "$port_path" ]
-		then
-			message_echo "WARNING: $pkg ($origin) is obsolete." >&2
-			message_echo >&2
-			continue
-		fi
-		if [ $is_reinstall_encouraged = no ]
-		then
-			if [ $opt_batch_mode = no ]
+		for origin in `pkg_info_flavored_origins "$pkg"`
+		do
+			[ -n "$origin" ] || continue
+			env LANG=C grep -q -Fx "$origin" "${DBDIR}/damaged_package" 2>/dev/null && continue
+			pkgsys_sanitychk_pkgcontents "$pkg" is_reinstall_encouraged && continue
+			port_path=`pkgsys_get_portpath_from_origin "$origin"`
+			if [ ! -d "$port_path" ]
 			then
-				message_echo "Do you want to reinstall it? (y/[n])"
-				message_query_yn_default_no || continue
+				message_echo "WARNING: $pkg ($origin) is obsolete." >&2
+				message_echo >&2
+				continue
 			fi
-		else
-			if [ $opt_batch_mode = no ]
+			if [ $is_reinstall_encouraged = no ]
 			then
-				message_echo "Do you want to reinstall it? ([y]/n)"
-				message_query_yn_default_yes || continue
+				if [ $opt_batch_mode = no ]
+				then
+					message_echo "Do you want to reinstall it? (y/[n])"
+					message_query_yn_default_no || continue
+				fi
+			else
+				if [ $opt_batch_mode = no ]
+				then
+					message_echo "Do you want to reinstall it? ([y]/n)"
+					message_query_yn_default_yes || continue
+				fi
+				database_record_reconf_recover_sanity "$origin"
 			fi
-			database_record_reconf_recover_sanity "$origin"
-		fi
+		done
 	done
 }
 
@@ -312,7 +314,7 @@ command_pkgs_packupgrade_create__manifest_reinst ()
 	PROGRAM_DEPENDS=COMMAND_PACKUPGRADE_PREPARE
 	_program_exec_restartable_loop_operation__routine ()
 	{
-		local origin dstdir tmp_manifest dbdir_new pkgname pkgarc timestamp_inst timestamp_arc origin_old
+		local origin dstdir tmp_manifest dbdir_new pkgname pkgarc timestamp_inst timestamp_arc origin_old origin_equiv
 		origin=$1
 		dstdir=${DBDIR}/command_packupgrade
 		tmp_manifest=${TMPDIR}/packupgrade::manifest
@@ -320,14 +322,14 @@ command_pkgs_packupgrade_create__manifest_reinst ()
 		dbdir_new=${DBDIR}/requires/$origin
 		pkgname=`pkgsys_get_installed_pkg_from_origin "$origin"`
 		[ -n "$pkgname" ] || return 0
-		if ! grep -Fxq "$pkgname" "$dstdir/etc/created_packages.lst" 2> /dev/null
+		if ! env LANG=C grep -Fxq "$pkgname" "$dstdir/etc/created_packages.lst" 2> /dev/null
 		then
 			if pkg_info_e "$pkgname"
 			then
 				if pkgarc=`pkgsys_pkgname_to_pkgarc "${PACKAGESDIR}" "$pkgname"`
 				then
 					timestamp_inst=`pkg_get_pkg_timestamp "$pkgname" 2> /dev/null` || :
-					timestamp_arc=`ls -lD %s "$pkgarc" 2> /dev/null | sed -E 's/[[:space:]]+/ /g' | cut -w -f 6 | grep -v '^$'` || :
+					timestamp_arc=`ls -lD %s "$pkgarc" 2> /dev/null | sed -E 's/[[:space:]]+/ /g' | cut -w -f 6 | env LANG=C grep -v '^$'` || :
 				else
 					timestamp_inst=1
 					timestamp_arc=0
@@ -340,10 +342,14 @@ command_pkgs_packupgrade_create__manifest_reinst ()
 						exit 1
 					fi
 				fi
-			elif grep -Fxq "$origin" ${DBDIR}/success.run.full.list
+			elif env LANG=C grep -Fxq "$origin" ${DBDIR}/success.run.full.list
 			then
-				message_echo "ERROR: Necessary package is missing [$origin]." >&2
-				exit 1
+				origin_equiv=`database_query_get_equivalent_orgin "$origin"`
+				if [ -z "$origin_equiv" ] || env LANG=C grep -Fxq "$origin_equiv" ${DBDIR}/success.run.full.list
+				then
+					message_echo "ERROR: Necessary package is missing [$origin]." >&2
+					exit 1
+				fi
 			fi
 			fileedit_add_a_line_if_new "$pkgname" "$dstdir/etc/created_packages.lst"
 		fi
