@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - Operations for recording operation results to the temporary database -
-# Copyright (C) 2013-2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2022 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -33,7 +33,7 @@ database_record_success ()
 				{
 					cat "$nodedir/requirements.$dbsuffix" 2> /dev/null
 					[ $level = full ] && cat "$nodedir/ignored_requirements.$tag" 2> /dev/null
-				} | grep -q -Fx -f "${DBDIR}/failed.list" 2> /dev/null
+				} | env LANG=C grep -q -Fx -f "${DBDIR}/failed.list" 2> /dev/null
 			then
 				fileedit_rm_a_line "$origin" "${DBDIR}/success.$dbsuffix.list"
 				fileedit_add_a_line_if_new "$origin" "${DBDIR}/todo_after_requirements_succeed.$dbsuffix.list"
