@@ -2,18 +2,18 @@
 # ==============================================================================
 # portsreinstall library script
 # - Common functions of main programs -
-# Copyright (C) 2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2018-2022 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
 # ============= Define the software version =============
 main_set_version ()
 {
-	MYVERSION=4.1.0
+	MYVERSION=4.1.1
 	COMPATIBLE_VERSIONS='^(4\.[1]\.[0-9])$'
 	# Template for development versions
-# 	MYVERSION=4.0.0+toward_4.1.0_20180920105217
-# 	COMPATIBLE_VERSIONS='^(4\.[0-1]\.[0-9]]|4\.[0]\.[0]+(|\+toward_4\.[0-1]\.[0-9]+_[0-9]+))$'
+# 	MYVERSION=4.1.0+toward_4.1.1_20220418123142
+# 	COMPATIBLE_VERSIONS='^(4\.[0-1]\.[0-9]]|4\.[0-1]\.[0]+(|\+toward_4\.[0-1]\.[0-9]+_[0-9]+))$'
 }
 
 # ============= Parse options, arguments and control parameters =============
@@ -523,7 +523,7 @@ main_reflect_conf_changes ()
 		if fileedit_manipulate_old_new_lines \
 			"${DBDIR}/conf.prev/setenv.sh" "${DBDIR}/conf/setenv.sh" "$tmpfile_old" "$tmpfile_new"
 		then
-			if grep -q -e ^LOCALBASE= -e ^LINUXBASE= -e ^PORTSDIR= "$tmpfile_old" "$tmpfile_new"
+			if env LANG=C grep -q -e ^LOCALBASE= -e ^LINUXBASE= -e ^PORTSDIR= "$tmpfile_old" "$tmpfile_new"
 			then
 				message_echo "ERROR: Migration of the temporary database is unavailable because LOCALBASE, LINUXBASE or PORTSDIR was changed." >&2
 				message_echo "        ${APPNAME} clean" >&2
