@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - Interface of libraries for command line options -
-# Copyright (C) 2013-2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2022 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -12,7 +12,7 @@ OPTIONS_SHIFT=0
 
 # ============= Database of options which are given at each run and not saved =============
 # [Syntax of option databases]
-# short_name, long_name, variable, defult_vaule, set_value
+# short_name, long_name, variable, default_value, set_value
 # Columns are delimited by tab characters.
 options_db_onetime ()
 {
@@ -48,9 +48,44 @@ options_db_saved_and_non_renewable ()
 eof
 }
 
+
+# ============= Definitions of value regulator functions =============
+options_db_value_regulators ()
+{
+	cat << eof
+eof
+}
+
 # ============= Regularize the option value =============
 options_regularize ()
 {
+	local nlines iline srcline var fnc
+	options_db_value_regulators > ${TMPDIR}/options_regularize::db
+	nlines=`wc -l < ${TMPDIR}/options_regularize::db`
+	iline=1
+	while [ $iline -le $nlines ]
+	do
+		srcline=`sed -n ${iline}p "${TMPDIR}/options_regularize::db"`
+		var=`echo "$srcline" | cut -f 1`
+		fnc=`echo "$srcline" | cut -f 2`
+		eval $var=\`$fnc \"\$$var\"\`
+		iline=$(($iline+1))
+	done
+}
+
+# ============= Regularize a value of an option value =============
+options_regularize_a_value ()
+{
+	local variable value fnc
+	variable=$1
+	value=$2
+	fnc=`options_db_value_regulators | grep "^$variable[[:space:]]" | cut -f 2`
+	if [ -n "$fnc" ]
+	then
+		eval $fnc "$value"
+	else
+		echo "$value"
+	fi
 }
 
 # ============= Database of all options to be saved =============
@@ -114,7 +149,7 @@ options_convert_shortname_to_varname ()
 {
 	local shortname
 	shortname=$1
-	options_db_all | grep -m 1 "^${shortname}[[:space:]]" | cut -f 3 | sort -u
+	options_db_all | env LANG=C grep -m 1 "^${shortname}[[:space:]]" | cut -f 3 | sort -u
 }
 
 # ============= Dump all command line arguments and options =============
@@ -144,7 +179,7 @@ options_set_default ()
 # ============= Get the argument for getopts =============
 options_getopts_arg ()
 {
-	options_db_all | cut -f 1,5 | grep -v '^[[:space:]]' | sort -u \
+	options_db_all | cut -f 1,5 | env LANG=C grep -v '^[[:space:]]' | sort -u \
 		| sed -E 's/[[:space:]]:$/:/;s/[[:space:]].*$//' | tr -d '\n'
 }
 
@@ -157,7 +192,7 @@ options_getopts_only_short ()
 	while getopts $getopts_option option > /dev/null
 	do
 		val=`str_escape_replaceval "$OPTARG"`
-		command=`options_db_all | grep -m 1 "^${option}[[:space:]]" \
+		command=`options_db_all | env LANG=C grep -m 1 "^${option}[[:space:]]" \
 			| cut -f 3,5 | sed -E 's/[[:space:]]/=/;s/:$/'$val'/' || :`
 		[ -n "$command" ] || return 1
 		echo -n "$command;OPTIND=$OPTIND;"
@@ -182,7 +217,7 @@ options_getopts ()
 		[ $# -eq 0 ] && break
 		erropt=$1
 		option=`expr "x$1" : '^x--\([a-zA-Z0-9][a-zA-Z0-9-]*\).*'` || break
-		command=`options_db_all | cut -f 2,3,5 | grep -m 1 "^${option}[[:space:]]" \
+		command=`options_db_all | cut -f 2,3,5 | env LANG=C grep -m 1 "^${option}[[:space:]]" \
 			| cut -f 2,3 | sed -E 's/[[:space:]]/=/' || :`
 		[ -n "$command" ] || break
 		if expr "$command" : '^[^=]*=:$' > /dev/null
@@ -208,21 +243,23 @@ options_getopts ()
 # ============= Filter option value definitions to pick up selected groups =============
 options_filter ()
 {
-	grep `options_db "$@" | cut -f 3 | sed 's/^/-e ^/;s/$/=/'`
+	env LANG=C grep `options_db "$@" | cut -f 3 | sed 's/^/-e ^/;s/$/=/'`
 }
 
 # ============= Compose an option set in a command line form from the current settings =============
 options_inverse_parse ()
 {
-	options_db "$@" | while read short_name long_name variable defult_vaule set_value
+	options_db "$@" | while read short_name long_name variable default_value set_value
 	do
+		eval default_value_eval=$default_value
+		default_value_regular=`options_regularize_a_value "$variable" "$default_value_eval"`
 		eval val=\$$variable
-		eval val_def=$defult_vaule
+		eval val_def=\$default_value_regular
 		eval val_set=$set_value
 		if [ "x$set_value" = x: ]
 		then
 			[ "x$val" = "x$val_def" ] \
-				|| misc_get_all_vardefs | grep -m 1 "^$variable=" \
+				|| misc_get_all_vardefs | env LANG=C grep -m 1 "^$variable=" \
 					| sed "s/^[^=]*=/-$short_name /"
 		elif [ "x$val" = "x$val_set" ]
 		then
@@ -237,13 +274,15 @@ options_filter_configured ()
 {
 	local tmpptn
 	tmpptn=${TMPDIR}/options_filter_configured:ptn
-	options_db "$@" | while read short_name long_name variable defult_vaule set_value
+	options_db "$@" | while read short_name long_name variable default_value set_value
 	do
+		eval default_value_eval=$default_value
+		default_value_regular=`options_regularize_a_value "$variable" "$default_value_eval"`
 		eval val=\$$variable
-		eval val_def=$defult_vaule
+		eval val_def=\$default_value_regular
 		[ "x$val" = "x$val_def" ] || echo "^$variable="
 	done > $tmpptn
-	grep -f "$tmpptn"
+	env LANG=C grep -f "$tmpptn"
 }
 
 # ============= Check inclusion of invalid options intending to be renewed =============
@@ -298,7 +337,7 @@ options_select_new_ports_if_duplicated ()
 	itemlist=${TMPDIR}/options_select_new_ports_if_duplicated::itemlist
 	cp /dev/null "$dstfile.tmp"
 	echo "$optargs" | sed -E 's/[ :]+/\
-/g' | grep -v '^$' | sort -u > $globs_list
+/g' | env LANG=C grep -v '^$' | sort -u > $globs_list
 	nlines=`wc -l < $globs_list`
 	iline=1
 	while [ $iline -le $nlines ]
@@ -318,7 +357,7 @@ options_select_new_ports_if_duplicated ()
 		do
 			if pkgsys_exists_or_existed_from_orig "$origin" \
 				|| cat "${DBDIR}/installed_ports" "${DBDIR}/targets_specified_so_far" 2> /dev/null \
-					| grep -q -Fx "$origin"
+					| env LANG=C grep -q -Fx "$origin"
 			then
 				echo "$origin" >> $tmp_existing_ports
 			else
