@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - Wrappers for hiding version differences in the Ports/Packages system -
-# Copyright (C) 2013-2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2022 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -229,7 +229,7 @@ pkgsys_fetch_legacy_remote ()
 	pkg_regexp=`str_escape_regexp "$pkg"`
 	pkgsys_ready_checksum_file || return
 	checksumpath=`pkgsys_ready_checksum_file__fetched_file`
-	validMD5=`grep -m 1 -E -e "^MD5[[:space:]]*\($pkg_regexp\.tbz\)[[:space:]]*=" "$checksumpath" | sed -E "s/^[^=]*=[[:space:]]*(.*)/\1/"`
+	validMD5=`env LANG=C grep -m 1 -E -e "^MD5[[:space:]]*\($pkg_regexp\.tbz\)[[:space:]]*=" "$checksumpath" | sed -E "s/^[^=]*=[[:space:]]*(.*)/\1/"`
 	if [ -z "$validMD5" ]
 	then
 		message_echo "WARNING: No check sum for $pkg.tbz." >&2
@@ -297,7 +297,7 @@ pkgsys_is_dependency_of_a_legacypkg_latest ()
 	rm -rf "$tmp_extract"
 	mkdir "$tmp_extract"
 	tar xf "$pkgarc" -C "$tmp_extract" +CONTENTS
-	grep -e '^@pkgdep[[:space:]]' -e '^@comment[[:space:]]*DEPORIGIN:' "$tmp_extract/+CONTENTS" \
+	env LANG=C grep -e '^@pkgdep[[:space:]]' -e '^@comment[[:space:]]*DEPORIGIN:' "$tmp_extract/+CONTENTS" \
 		| sed 's/^@pkgdep[[:space:]]*//;s/^@comment[[:space:]]*DEPORIGIN://' > $tmp_contents
 	nlines=`wc -l < $tmp_contents`
 	iline=1
@@ -438,28 +438,37 @@ pkgsys_def_pkgtools ()
 			# Return the total storage space occupied by the installed files in bytes
 			pkg info -qs "$@" 2> /dev/null | sed 's/KiB$/*1024/;s/MiB$/*1024^2/;s/GiB$/*1024^3/;s/B$//' | tr '\n' + | sed 's/+$//' | bc -l
 		}
-		pkg_info_flavor ()
+		pkg_info_flavors ()
+		# For an unflavored pattern, all matching flavors are returned.
 		{
 			local glob_unflavored
 			glob_unflavored=$1
-			pkg query -g '%At\t%Av' "$glob_unflavored" 2> /dev/null | grep -E '^flavor[[:space:]]' | cut -f 2
+			pkg query -g '%At\t%Av' "$glob_unflavored" 2> /dev/null | env LANG=C grep -E '^flavor[[:space:]]' | cut -f 2
 		}
-		pkg_info_flavored_origin ()
+		pkg_info_flavored_origins ()
+		# For an unflavored pattern, all matching flavors are returned.
 		{
 			local glob_unflavored origin_unflavored
 			glob_unflavored=$1
 			origin_unflavored=`pkg_info_qo "$glob_unflavored" 2> /dev/null || :`
-			flavor=`pkg_info_flavor "$glob_unflavored" 2> /dev/null || :`
-			[ -z "$flavor" ] || flavor=@$flavor
-			echo "$origin_unflavored$flavor"
+			flavors=`pkg_info_flavors "$glob_unflavored" 2> /dev/null || :`
+			if [ -z "$flavors" ]
+			then
+				echo "$origin_unflavored"
+			else
+				for flavor in $flavors
+				do
+					echo "$origin_unflavored@$flavor"
+				done
+			fi
 		}
 		pkg_info_all_flavored_origins ()
 		{
 			local tmp_flavored tmp_flavored_ptn
 			tmp_flavored_ptn=${TMPDIR}/pkg_info_all_flavored_origins:flavored_ptn
-			pkg query '%o\t%At\t%Av' 2> /dev/null | grep -E '^[^[:space:]]+[[:space:]]flavor[[:space:]]' | cut -f 1,3 | tr '\t' @ | sed 's/@$//'
-			pkg query '%n\t%At' 2> /dev/null | grep -E '^[^[:space:]]+[[:space:]]flavor$' | cut -f 1 > $tmp_flavored_ptn
-			pkg query '%n\t%o' 2> /dev/null | grep -vFx -f "$tmp_flavored_ptn" | cut -f 2
+			pkg query '%o\t%At\t%Av' 2> /dev/null | env LANG=C grep -E '^[^[:space:]]+[[:space:]]flavor[[:space:]]' | cut -f 1,3 | tr '\t' @ | sed 's/@$//'
+			pkg query '%n\t%At' 2> /dev/null | env LANG=C grep -E '^[^[:space:]]+[[:space:]]flavor$' | cut -f 1 > $tmp_flavored_ptn
+			pkg query '%n\t%o' 2> /dev/null | env LANG=C grep -vFx -f "$tmp_flavored_ptn" | cut -f 2
 		}
 		pkg_check_sanity ()
 		{
@@ -469,9 +478,9 @@ pkgsys_def_pkgtools ()
 			tmp_stderr=${TMPDIR}/pkgng:pkg_check_sanity:stderr
 			pkg check -s "$pkg" > $tmp_stdout 2> $tmp_stderr || :
 			{
-				grep '^[^:]*: checksum mismatch for ' "$tmp_stderr" | sed 's/^[^:]*: checksum mismatch for //' || :
-				grep '^[^:]*: missing file ' "$tmp_stderr" | sed 's/^[^:]*: missing file //' || :
-				if grep -q '^pkg: .*: No such file or directory$' "$tmp_stderr"	# For the old specification of pkg(8)
+				env LANG=C grep '^[^:]*: checksum mismatch for ' "$tmp_stderr" | sed 's/^[^:]*: checksum mismatch for //' || :
+				env LANG=C grep '^[^:]*: missing file ' "$tmp_stderr" | sed 's/^[^:]*: missing file //' || :
+				if env LANG=C grep -q '^pkg: .*: No such file or directory$' "$tmp_stderr"	# For the old specification of pkg(8)
 				then
 					pkg info -ql "$pkg" 2> /dev/null | while read filepath
 					do
@@ -488,11 +497,14 @@ pkgsys_def_pkgtools ()
 		}
 		pkg_info_gen_pkg_origin_table ()
 		{
+			local pkgname origin
 			#	pkg query -g '%n-%v\t%o' \* 2> /dev/null > ${DBDIR}/installed_ports:pkg_vs_origin.tbl
-			pkg info -qa 2> /dev/null | while read pkgname
+			for pkgname in `pkg info -qa 2> /dev/null`
 			do
-				origin=`pkg_info_flavored_origin "$pkgname"`
-				printf '%s\t%s\n' "$pkgname" "$origin"
+				for origin in `pkg_info_flavored_origins "$pkgname"`
+				do
+					printf '%s\t%s\n' "$pkgname" "$origin"
+				done
 			done
 		}
 		pkg_create_b ()
@@ -720,16 +732,16 @@ pkgsys_def_pkgtools ()
 			PKGNG_AUTODEPS=NO
 			PKGNG_PORTAUDIT_SITE='http=//portaudit.FreeBSD.org/auditfile.tbz'
 			# Load configuration for pkg(1)
-			pkg_conf=`pkg query %Fp pkg 2> /dev/null | grep '/etc/pkg\.conf\.sample$' | sed 's/\.sample$//'` || :
+			pkg_conf=`pkg query %Fp pkg 2> /dev/null | env LANG=C grep '/etc/pkg\.conf\.sample$' | sed 's/\.sample$//'` || :
 			[ -n "$pkg_conf" ] || pkg_conf=${MYPREFIX}/etc/pkg.conf
-			grep -v -e '^[[:space:]]*#' -e '^[[:space:]]*$' "$pkg_conf" 2> /dev/null \
-				| grep -e '^[[:space:]]*[A-Z0-9_]*[[:space:]]*:[[:space:]]*.*' \
+			env LANG=C grep -v -e '^[[:space:]]*#' -e '^[[:space:]]*$' "$pkg_conf" 2> /dev/null \
+				| env LANG=C grep -e '^[[:space:]]*[A-Z0-9_]*[[:space:]]*:[[:space:]]*.*' \
 				| while read srcline
 			do
 				var=`expr "$srcline" : '^[[:space:]]*\([A-Z0-9_]*\)[[:space:]]*:.*'` || :
 				val=`expr "$srcline" : '^[[:space:]]*[A-Z0-9_]*[[:space:]]*:[[:space:]]*\(.*\)'` || :
 				eval PKGNG_$var=\$val
-				misc_get_all_vardefs | grep "^PKGNG_$var="
+				misc_get_all_vardefs | env LANG=C grep "^PKGNG_$var="
 			done > ${TMPDIR}/pkgsys_def_pkgtools:pkg.conf.sh
 			. "${TMPDIR}/pkgsys_def_pkgtools:pkg.conf.sh"
 		}
@@ -746,10 +758,10 @@ pkgsys_def_pkgtools ()
 			message_echo "INFO: Installing pkgng by legacy package tool."
 			checksumpath=`pkgsys_ready_checksum_file__fetched_file`
 			pkgname=`sed 's/^MD5[[:space:]]*(//;s/\.tbz)[[:space:]]*=[^=]*$//' "$checksumpath" \
-				| grep -m 1 '^pkg-[0-9]'` || :
+				| env LANG=C grep -m 1 '^pkg-[0-9]'` || :
 			[ -n "$pkgname" ] && pkg_inst_remote_wild "$pkgname" && return
 			message_echo "INFO: Failed by package, so installing pkgng by port."
-			grep -Ev '^[[:space:]]*WITH_PKG(|NG)=' /etc/make.conf > ${TMPDIR}/make.conf 2> /dev/null || :
+			env LANG=C grep -Ev '^[[:space:]]*WITH_PKG(|NG)=' /etc/make.conf > ${TMPDIR}/make.conf 2> /dev/null || :
 			echo WITHOUT_PKG=yes >> ${TMPDIR}/make.conf
 			echo WITHOUT_PKGNG=yes >> ${TMPDIR}/make.conf
 			dev_out=/dev/stdout
@@ -851,7 +863,7 @@ pkgsys_def_pkgtools ()
 		}
 		pkg_info_qR ()
 		{
-			pkg_info -qR "$@" 2> /dev/null | grep -v '^$'
+			pkg_info -qR "$@" 2> /dev/null | env LANG=C grep -v '^$'
 		}
 		pkg_info_Ex ()
 		{
@@ -866,12 +878,12 @@ pkgsys_def_pkgtools ()
 			# Return the total storage space occupied by the installed files in bytes
 			pkg_info -qs "$@" 2> /dev/null | sed 's/[^0-9]*/*1024/' | tr '\n' + | sed 's/+$//' | bc -l
 		}
-		pkg_info_flavor ()
+		pkg_info_flavors ()
 		{
 			pkg_info -qe "$@" 2> /dev/null && echo
 			:
 		}
-		pkg_info_flavored_origin ()
+		pkg_info_flavored_origins ()
 		{
 			local glob_unflavored tmp_stdout
 			glob_unflavored=$1
@@ -1105,15 +1117,22 @@ pkgsys_get_installed_pkg_from_origin ()
 	origin=$1
 	origin_unflavored=`pkgsys_get_unflavored_origin "$origin"`
 	flavor_origin=`pkgsys_get_flavor_from_origin "$origin"`
-	pkg_info_qO "$origin_unflavored" 2> /dev/null | while read pkgname
-	do
-		flavor_pkg=`pkg_info_flavor "$pkgname"`
-		if [ "x$flavor_origin" = "x$flavor_pkg" ]
-		then
-			echo "$pkgname"
-			break
-		fi
-	done
+	if [ -n "$flavor_origin" ]
+	then
+		for pkgname in `pkg_info_qO "$origin_unflavored" 2> /dev/null`
+		do
+			for flavor_pkg in `pkg_info_flavors "$pkgname"`
+			do
+				if [ "x$flavor_origin" = "x$flavor_pkg" ]
+				then
+					echo "$pkgname"
+					return
+				fi
+			done
+		done
+	else
+		pkg_info_qO "$origin_unflavored" 2> /dev/null
+	fi
 	:
 }
 
@@ -1123,6 +1142,7 @@ pkgsys_get_installed_pkg_from_glob ()
 	local glob regexp
 	for glob in "$@"
 	do
+		[ -z "$glob" ] && continue
 		if regexp=`expr "$glob" : ':\(.*\)'`
 		then
 			pkg_info_Ex "$regexp"
@@ -1140,8 +1160,11 @@ pkgsys_exists_from_orig ()
 	origin_unflavored=`pkgsys_get_unflavored_origin "$origin"`
 	pkg_info_eO "$origin_unflavored" 2> /dev/null || return
 	flavor_origin=`pkgsys_get_flavor_from_origin "$origin"`
-	flavor_pkg=`pkg_info_flavor "$origin_unflavored"`
-	[ "x$flavor_origin" = "x$flavor_pkg" ]
+	for flavor_pkg in `pkg_info_flavors "$origin_unflavored"`
+	do
+		[ "x$flavor_origin" = "x$flavor_pkg" ] && return
+	done
+	false
 }
 
 # ============= Generate the package names vs origins table at the initial state =============
@@ -1158,7 +1181,7 @@ pkgsys_exists_or_existed_from_orig ()
 	local origin
 	origin=$1
 	cut -f 2 "${DBDIR}/installed_ports:pkg_vs_origin.tbl" 2> /dev/null \
-		| grep -q -Fx "$origin" || pkgsys_exists_from_orig "$origin"
+		| env LANG=C grep -q -Fx "$origin" || pkgsys_exists_from_orig "$origin"
 }
 
 # ============= Get the name of an initially installed package for a flavored origin =============
@@ -1177,14 +1200,16 @@ pkgsys_get_init_pkg_from_orig ()
 	else
 		origin_unflavored=`pkgsys_get_unflavored_origin "$origin"`
 		flavor_origin=`pkgsys_get_flavor_from_origin "$origin"`
-		pkg_info_qO "$origin_unflavored" 2> /dev/null | while read pkgname
+		for pkgname in `pkg_info_qO "$origin_unflavored" 2> /dev/null`
 		do
-			flavor_pkg=`pkg_info_flavor "$pkgname"`
-			if [ "x$flavor_origin" = "x$flavor_pkg" ]
-			then
-				echo "$pkgname"
-				break
-			fi
+			for flavor_pkg in `pkg_info_flavors "$pkgname"`
+			do
+				if [ "x$flavor_origin" = "x$flavor_pkg" ]
+				then
+					echo "$pkgname"
+					return
+				fi
+			done
 		done
 	fi
 	:
@@ -1199,9 +1224,10 @@ pkgsys_get_my_current_pkg ()
 # ============= Get the flavored origin of this utility =============
 pkgsys_get_my_origin ()
 {
-	pkgsys_get_my_current_pkg | while read pkgname
+	local pkgname
+	for pkgname in `pkgsys_get_my_current_pkg`
 	do
-		pkg_info_flavored_origin "$pkgname"
+		pkg_info_flavored_origins "$pkgname"
 	done
 }
 
@@ -1210,9 +1236,9 @@ pkgsys_init_pkg_orig_by_ambiguous_matching ()
 {
 	local pkg origin tmporigin ambsuffix len_pkg pkg_regexp norigins
 	pkg=$1
-	origin=`pkg_info_flavored_origin "$pkg"`
-	[ -n "$origin" ] && { echo "$origin"; return; }
 	tmporigin=${TMPDIR}/pkgsys_init_pkg_orig_by_ambiguous_matching::origin
+	pkg_info_flavored_origins "$pkg" > $tmporigin
+	[ `wc -l < $tmporigin` -gt 0 ] && { cat "$tmporigin"; return; }
 	ambsuffix=
 	norigins=0
 	len_pkg=`echo -n "$pkg" | wc -c`
@@ -1221,7 +1247,7 @@ pkgsys_init_pkg_orig_by_ambiguous_matching ()
 		while :
 		do
 			pkg_regexp=`str_escape_regexp "$pkg"`$ambsuffix
-			grep -E "^${pkg_regexp}[[:space:]]" "${DBDIR}/installed_ports:pkg_vs_origin.tbl" 2> /dev/null \
+			env LANG=C grep -E "^${pkg_regexp}[[:space:]]" "${DBDIR}/installed_ports:pkg_vs_origin.tbl" 2> /dev/null \
 				| cut -f 2 > $tmporigin
 			norigins=`wc -l < $tmporigin`
 			[ $norigins -gt 0 ] && break
@@ -1298,23 +1324,23 @@ pkgsys_eval_ports_glob ()
 				glob_regexp_unflavored=`expr "$glob_regexp" : '\([^@]*\)' || :`$
 				glob_regexp_flavor=^`expr "$glob_regexp" : '[^@]*@\([^@]*\)' || :`
 					
-				grep -E "$glob_regexp_unflavored" "$unflavored_origlist" 2>&1 | while read origin_unflavored
+				env LANG=C grep -E "$glob_regexp_unflavored" "$unflavored_origlist" 2>&1 | while read origin_unflavored
 				do
 					fs_fix_unionfs_image_if_hidden "${PORTSDIR}/$origin_unflavored"
 					make -C "${PORTSDIR}/$origin_unflavored" -V FLAVORS 2> /dev/null | \
-						tr '[:space:]' '\n' | grep -v '^$' | grep -E "$glob_regexp_flavor" | sed -E "s|^|$origin_unflavored@|"
+						tr '[:space:]' '\n' | env LANG=C grep -v '^$' | env LANG=C grep -E "$glob_regexp_flavor" | sed -E "s|^|$origin_unflavored@|"
 				done
 				{
 					pkg_info_all_flavored_origins
 					cut -f 2 "${DBDIR}/installed_ports:pkg_vs_origin.tbl" 2> /dev/null
-				} | grep -E "$glob_regexp" 2>&1 || :
+				} | env LANG=C grep -E "$glob_regexp" 2>&1 || :
 			else
-				grep -E "$glob_regexp" "$unflavored_origlist" 2>&1 | while read origin_unflavored
+				env LANG=C grep -E "$glob_regexp" "$unflavored_origlist" 2>&1 | while read origin_unflavored
 				do
 					origin_unflavored_rpl=`str_escape_replaceval "$origin_unflavored"`
 					fs_fix_unionfs_image_if_hidden "${PORTSDIR}/$origin_unflavored"
 					make -C "${PORTSDIR}/$origin_unflavored" -V FLAVORS 2> /dev/null | \
-						tr '[:space:]' '\n' | grep -v '^$' > $tmp_flavors || :
+						tr '[:space:]' '\n' | env LANG=C grep -v '^$' > $tmp_flavors || :
 					if [ `wc -l < $tmp_flavors` -gt 0 ]
 					then
 						sed -E "s|^|$origin_unflavored_rpl@|" "$tmp_flavors"
@@ -1327,7 +1353,7 @@ pkgsys_eval_ports_glob ()
 			{
 				pkg_info_all_flavored_origins
 				cut -f 2 "${DBDIR}/installed_ports:pkg_vs_origin.tbl" 2> /dev/null
-			} | grep -E "$glob_regexp_allflavors" 2>&1 || :
+			} | env LANG=C grep -E "$glob_regexp_allflavors" 2>&1 || :
 		else
 			if expr "$glob" : '[a-z][a-zA-Z0-9_.+-]*[a-zA-Z0-9_.+]$' > /dev/null 2>&1 && \
 				[ `expr "$glob" : '.*-[0-9]' 2>&1` -eq 0 ]
@@ -1336,7 +1362,7 @@ pkgsys_eval_ports_glob ()
 			else
 				glob_regexp2=$glob_regexp
 			fi
-			grep -n -E "$glob_regexp2" "$pkglist" 2>&1 | cut -d : -f 1 \
+			env LANG=C grep -n -E "$glob_regexp2" "$pkglist" 2>&1 | cut -d : -f 1 \
 				| while read index
 			do
 				sed -n ${index}p "$unflavored_origlist"
@@ -1345,7 +1371,7 @@ pkgsys_eval_ports_glob ()
 				origin_unflavored_rpl=`str_escape_replaceval "$origin_unflavored"`
 				fs_fix_unionfs_image_if_hidden "${PORTSDIR}/$origin_unflavored"
 				make -C "${PORTSDIR}/$origin_unflavored" -V FLAVORS 2> /dev/null | \
-					tr '[:space:]' '\n' | grep -v '^$' > $tmp_flavors || :
+					tr '[:space:]' '\n' | env LANG=C grep -v '^$' > $tmp_flavors || :
 				[ `wc -l < $tmp_flavors` -gt 0 ] || echo > $tmp_flavors
 				sed -E "s/^/$origin_unflavored_rpl /" "$tmp_flavors"
 			done | while read origin_unflavored flavor
@@ -1353,24 +1379,24 @@ pkgsys_eval_ports_glob ()
 				if [ -n "$flavor" ]
 				then
 					if make -C "${PORTSDIR}/$origin_unflavored" package-name FLAVOR=$flavor 2> /dev/null | \
-					grep -qE "$glob_regexp2"
+					env LANG=C grep -qE "$glob_regexp2"
 					then
 						echo "$origin_unflavored@$flavor"
 					fi
 				else
 					if make -C "${PORTSDIR}/$origin_unflavored" package-name 2> /dev/null | \
-					grep -qE "$glob_regexp2"
+					env LANG=C grep -qE "$glob_regexp2"
 					then
 						echo "$origin_unflavored"
 					fi
 				fi
 			done || :
 			glob_regexp2=`echo "$glob_regexp" | sed -E 's/\$*$//' 2>&1 || :`'[[:space:]]'
-			grep -E "$glob_regexp2" "${DBDIR}/installed_ports:pkg_vs_origin.tbl" 2> /dev/null \
+			env LANG=C grep -E "$glob_regexp2" "${DBDIR}/installed_ports:pkg_vs_origin.tbl" 2> /dev/null \
 				| cut -f 2 || :
-			pkg_info_Ex "$glob_regexp" | while read pkgname
+			for pkgname in `pkg_info_Ex "$glob_regexp"`
 			do
-				pkg_info_flavored_origin "$pkgname"
+				pkg_info_flavored_origins "$pkgname"
 			done
 		fi
 	done | sort -u
@@ -1384,45 +1410,47 @@ pkgsys_create_backup_pkg ()
 	dstdir=$2
 	rm -rf "${TMPDIR}"/package.tmp
 	mkdir "${TMPDIR}"/package.tmp
-	origin=`pkg_info_flavored_origin "$pkgname"`
-	if backup_pkg_old=`pkgsys_get_backup_pkg "$origin"` && \
-		[ ! -e "${DBDIR}/requires/$origin/installed_timestamp" -o \
-			"$backup_pkg_old" -nt "${DBDIR}/requires/$origin/installed_timestamp" ]
-	then
-		echo "$backup_pkg_old"
-		return
-	fi
-	if ( cd "${TMPDIR}"/package.tmp && pkg_create_b "$pkgname" > /dev/null )
-	then
-		pkgname_ptn=`str_escape_regexp "$pkgname"`
-		backup_pkg=`ls "${TMPDIR}"/package.tmp | \
-			grep -m 1 -E "^${pkgname_ptn}\.(txz|tbz|tgz|tar)$"` || :
-	fi
-	if [ -z "$backup_pkg" ]
-	then
-		message_echo "WARNING: Failed to create backup package for $pkgname." >&2
-		return 1
-	fi
-	dbpath=${DBDIR}/backup/$origin
-	mkdir -p "$dbpath"
-	pkg_info_qL "$pkgname" > $dbpath/previously_installed_files
-	mkdir -p "$dstdir"
-	mv "${TMPDIR}/package.tmp/$backup_pkg" "$dstdir"
-	pkgpath=$dstdir/$backup_pkg
-	cat "${DBDIR}/backup_pkgarcs.lst" 2> /dev/null | \
-		while read origin_cur pkgpath_cur
-		do
-			[ "$pkgpath_cur" = "$pkgpath" ] && continue
-			if [ "$origin_cur" = "$origin" ]
-			then
-				rm -f "$pkgpath_cur"
-			else
-				printf '%s\t%s\n' "$origin_cur" "$pkgpath_cur"
-			fi
-		done > ${DBDIR}/backup_pkgarcs.lst.tmp
-	printf '%s\t%s\n' "$origin" "$pkgpath" >> ${DBDIR}/backup_pkgarcs.lst.tmp
-	mv "${DBDIR}/backup_pkgarcs.lst.tmp" "${DBDIR}/backup_pkgarcs.lst"
-	echo "$pkgpath"
+	for origin in `pkg_info_flavored_origins "$pkgname"`
+	do
+		if backup_pkg_old=`pkgsys_get_backup_pkg "$origin"` && \
+			[ ! -e "${DBDIR}/requires/$origin/installed_timestamp" -o \
+				"$backup_pkg_old" -nt "${DBDIR}/requires/$origin/installed_timestamp" ]
+		then
+			echo "$backup_pkg_old"
+			return
+		fi
+		if ( cd "${TMPDIR}"/package.tmp && pkg_create_b "$pkgname" > /dev/null )
+		then
+			pkgname_ptn=`str_escape_regexp "$pkgname"`
+			backup_pkg=`ls "${TMPDIR}"/package.tmp | \
+				env LANG=C grep -m 1 -E "^${pkgname_ptn}\.(txz|tbz|tgz|tar|pkg)$"` || :
+		fi
+		if [ -z "$backup_pkg" ]
+		then
+			message_echo "WARNING: Failed to create backup package for $pkgname." >&2
+			return 1
+		fi
+		dbpath=${DBDIR}/backup/$origin
+		mkdir -p "$dbpath"
+		pkg_info_qL "$pkgname" > $dbpath/previously_installed_files
+		mkdir -p "$dstdir"
+		mv "${TMPDIR}/package.tmp/$backup_pkg" "$dstdir"
+		pkgpath=$dstdir/$backup_pkg
+		cat "${DBDIR}/backup_pkgarcs.lst" 2> /dev/null | \
+			while read origin_cur pkgpath_cur
+			do
+				[ "$pkgpath_cur" = "$pkgpath" ] && continue
+				if [ "$origin_cur" = "$origin" ]
+				then
+					rm -f "$pkgpath_cur"
+				else
+					printf '%s\t%s\n' "$origin_cur" "$pkgpath_cur"
+				fi
+			done > ${DBDIR}/backup_pkgarcs.lst.tmp
+		printf '%s\t%s\n' "$origin" "$pkgpath" >> ${DBDIR}/backup_pkgarcs.lst.tmp
+		mv "${DBDIR}/backup_pkgarcs.lst.tmp" "${DBDIR}/backup_pkgarcs.lst"
+		echo "$pkgpath"
+	done
 }
 
 # ============= Delete a back-up package archive for a flavored port origin =============
@@ -1431,12 +1459,12 @@ pkgsys_delete_backup_pkg ()
 	local origin origin_regexp
 	origin=$1
 	origin_regexp=`str_escape_regexp "$origin"`
-	grep -E "^${origin_regexp}[[:space:]]" "${DBDIR}/backup_pkgarcs.lst" 2> /dev/null \
+	env LANG=C grep -E "^${origin_regexp}[[:space:]]" "${DBDIR}/backup_pkgarcs.lst" 2> /dev/null \
 		| cut -f 2 | while read pkgpath_cur
 		do
 			rm -f "$pkgpath_cur"
 		done
-	grep -v -E "^${origin_regexp}[[:space:]]" "${DBDIR}/backup_pkgarcs.lst" \
+	env LANG=C grep -v -E "^${origin_regexp}[[:space:]]" "${DBDIR}/backup_pkgarcs.lst" \
 		2> /dev/null > ${DBDIR}/backup_pkgarcs.lst.tmp || :
 	mv "${DBDIR}/backup_pkgarcs.lst.tmp" "${DBDIR}/backup_pkgarcs.lst"
 }
@@ -1449,7 +1477,7 @@ pkgsys_get_backup_pkg ()
 	tmpnewest=${TMPDIR}/pkgsys_get_backup_pkg::newest
 	origin_regexp=`str_escape_regexp "$origin"`
 	rm -f "$tmpnewest"
-	grep -E "^${origin_regexp}[[:space:]]" "${DBDIR}/backup_pkgarcs.lst" 2> /dev/null \
+	env LANG=C grep -E "^${origin_regexp}[[:space:]]" "${DBDIR}/backup_pkgarcs.lst" 2> /dev/null \
 		| cut -f 2 | while read pkgpath
 	do
 		pkgpath_newest=`cat "$tmpnewest" 2> /dev/null` || :
@@ -1475,7 +1503,7 @@ pkgsys_chk_match_to_restored_files_by_backup_pkg ()
 	origin=$1
 	filelist=$2
 	dbfile=${DBDIR}/backup/$origin/previously_installed_files
-	grep -qFx -f "$filelist" "$dbfile" 2> /dev/null
+	env LANG=C grep -qFx -f "$filelist" "$dbfile" 2> /dev/null
 }
 
 # ============= Get a package name from a package archive file name =============
@@ -1483,7 +1511,7 @@ pkgsys_pkgarc_to_pkgname ()
 {
 	local pkgfile_path
 	pkgfile_path=$1
-	basename "$pkgfile_path" | sed -E 's/\.(txz|tbz|tgz|tar)$//'
+	basename "$pkgfile_path" | sed -E 's/\.(txz|tbz|tgz|tar|pkg)$//'
 }
 
 # ============= Get the file name of an existing package archive for a package name =============
@@ -1497,7 +1525,7 @@ pkgsys_pkgname_to_pkgarc ()
 	if [ "x$PKGSYS_USE_PKGNG" = xyes ]
 	then
 		pkgname_ptn=`str_escape_regexp "$pkgname"`
-		pkgnode=`ls "$pkgdir" 2> /dev/null | grep -m 1 -E "^${pkgname_ptn}\.(txz|tbz|tgz|tar)$"` || :
+		pkgnode=`ls "$pkgdir" 2> /dev/null | env LANG=C grep -m 1 -E "^${pkgname_ptn}\.(txz|tbz|tgz|tar|pkg)$"` || :
 	elif [ -e "$pkgdir/$pkgname.tbz" ]
 	then
 		pkgnode=$pkgname.tbz
@@ -1513,8 +1541,8 @@ pkgsys_eval_ports_glob_even_if_nonexistent ()
 	glob_pattern=$1
 	{
 		pkgsys_eval_ports_glob "$glob_pattern" 2> /dev/null || :
-		echo "$glob_pattern" | grep -E '^[a-z]+/[a-zA-Z0-9_.+-]+(|@[a-zA-Z0-9_.+-]+)$' || :
-	} | grep -v -e '^$' | sort -u
+		echo "$glob_pattern" | env LANG=C grep -E '^[a-z]+/[a-zA-Z0-9_.+-]+(|@[a-zA-Z0-9_.+-]+)$' || :
+	} | env LANG=C grep -v -e '^$' | sort -u
 }
 
 # ============= Evaluate glob patterns and add/remove non-existing/existing ones of them to/from a file =============
@@ -1527,7 +1555,7 @@ pkgsys_register_evaluated_globs ()
 	dirpath=`dirname "$listpath"`
 	tmp_evaluated=${TMPDIR}/pkgsys_register_evaluated_globs:pkgsys_eval_ports_glob
 	echo "$@" | sed -E 's/[ :]+/\
-/g' | grep -v '^$' | sort -u | while read -r glob
+/g' | env LANG=C grep -v '^$' | sort -u | while read -r glob
 	do
 		pkgsys_eval_ports_glob "$glob" > $tmp_evaluated
 		[ `wc -l < $tmp_evaluated` -ge 1 ] || \
@@ -1583,7 +1611,7 @@ pkgsys_get_conflicting_pkgs_patterns ()
 		conflicts_config=
 		;;
 	esac
-	echo "$conflicts $conflicts_makevar $conflicts_config" | tr ' ' '\n' | grep -v '^$' | sort -u
+	echo "$conflicts $conflicts_makevar $conflicts_config" | tr ' ' '\n' | env LANG=C grep -v '^$' | sort -u
 }	
 
 # ============= Get conflicting installed packages of a port =============
@@ -1611,7 +1639,7 @@ pkgsys_chk_conflict_by_a_pkg ()
 	tmp_conflicts_ptn=${TMPDIR}/pkgsys_chk_conflict_by_a_pkg::conflicts_ptn
 	pkgsys_get_conflicting_pkgs_patterns "$mode" "$origin" \
 		| str_convert_glob_to_regexp_pattern > $tmp_conflicts_ptn
-	echo "$pkg" | grep -q -E -f "$tmp_conflicts_ptn"
+	echo "$pkg" | env LANG=C grep -q -E -f "$tmp_conflicts_ptn"
 }	
 
 # ============= Check whether installed files are lost or broken for a package =============
@@ -1642,7 +1670,7 @@ pkgsys_sanitychk_pkgcontents ()
 			break
 		fi
 		filename_esc=`str_escape_regexp "$filename"`
-		if file "$filename" | sed -E "s/^$filename_esc:[[:space:]]//" | grep -q '^ELF '
+		if file "$filename" | sed -E "s/^$filename_esc:[[:space:]]//" | env LANG=C grep -q '^ELF '
 		then
 			_is_reinstall_encouraged=yes
 			break
@@ -1655,35 +1683,37 @@ pkgsys_sanitychk_pkgcontents ()
 		fi
 	done
 	eval "$var_is_reinstall_encouraged=\$_is_reinstall_encouraged"
-	origin=`pkg_info_flavored_origin "$pkg"`
-	if [ $opt_batch_mode = no ]
-	then
-		message_echo "[$pkg ($origin)]"
-		sed 's/^/  /' "$tmp_sanity"
-		message_echo
-	else
-		pkg_replace=`str_escape_replaceval "$pkg"`
-		origin_replace=`str_escape_replaceval "$origin"`
-		sed "s/^/$pkg_replace\\\\$origin_replace\\\\$_is_reinstall_encouraged\\\\/" "$tmp_sanity" | tr '\\' '\t'
-	fi
+	for origin in `pkg_info_flavored_origins "$pkg"`
+	do
+		if [ $opt_batch_mode = no ]
+		then
+			message_echo "[$pkg ($origin)]"
+			sed 's/^/  /' "$tmp_sanity"
+			message_echo
+		else
+			pkg_replace=`str_escape_replaceval "$pkg"`
+			origin_replace=`str_escape_replaceval "$origin"`
+			sed "s/^/$pkg_replace\\\\$origin_replace\\\\$_is_reinstall_encouraged\\\\/" "$tmp_sanity" | tr '\\' '\t'
+		fi
+	done
 	return 1
 }
 
 # ============= Check whether the port options database is once saved =============
-pkgsys_exists_saved_port_oprions_timestamps ()
+pkgsys_exists_saved_port_options_timestamps ()
 {
 	[ -d "${DBDIR}/ls_dbdir" ]
 }
 
 # ============= Get the current all timestamp information of port options =============
-pkgsys_get_current_port_oprions_timestamp ()
+pkgsys_get_current_port_options_timestamp ()
 {
 	local portdb_needle_regexp
 	portdb_needle_regexp=`str_escape_regexp "$1"`
 	{
 		ls -lD %Y%m%d:%H%M%S "${PORT_DBDIR}" | if [ -n "$portdb_needle_regexp" ]
 		then
-			grep -E "[[:space:]]$portdb_needle_regexp$" || :
+			env LANG=C grep -E "[[:space:]]$portdb_needle_regexp$" || :
 		else
 			cat
 		fi
@@ -1699,20 +1729,20 @@ pkgsys_get_current_port_oprions_timestamp ()
 }
 
 # ============= Get the saved all timestamp information of port options =============
-pkgsys_get_saved_port_oprions_timestamps_all ()
+pkgsys_get_saved_port_options_timestamps_all ()
 {
 	mkdir -p "${DBDIR}/ls_dbdir"
 	cat "${DBDIR}/ls_dbdir/"*.log 2> /dev/null || :
 }
 
 # ============= Convert a list of port origins to port options timestamp log names =============
-pkgsys_conv_portorigin_to_port_oprion_timestamp_logname ()
+pkgsys_conv_portorigin_to_port_option_timestamp_logname ()
 {
 	sed 's|/|_|'
 }
 
 # ============= Get the file name of the port options database of a port =============
-pkgsys_get_port_oprion_database ()
+pkgsys_get_port_option_database ()
 {
 	local origin
 	origin=$1
@@ -1725,30 +1755,30 @@ pkgsys_get_port_oprion_database ()
 }
 
 # ============= Save the timestamp information of port options of a port =============
-pkgsys_save_port_oprion_timestamp ()
+pkgsys_save_port_option_timestamp ()
 {
 	local origin portoptlog portoptdb
 	origin=$1
-	portoptlog=`echo "$origin" | pkgsys_conv_portorigin_to_port_oprion_timestamp_logname`
-	portoptdb=`pkgsys_get_port_oprion_database "$origin"`
+	portoptlog=`echo "$origin" | pkgsys_conv_portorigin_to_port_option_timestamp_logname`
+	portoptdb=`pkgsys_get_port_option_database "$origin"`
 	mkdir -p "${DBDIR}/ls_dbdir"
-	pkgsys_get_current_port_oprions_timestamp "$portoptdb" > ${DBDIR}/ls_dbdir/$portoptlog.log 2> /dev/null || :
+	pkgsys_get_current_port_options_timestamp "$portoptdb" > ${DBDIR}/ls_dbdir/$portoptlog.log 2> /dev/null || :
 }
 
 # ============= Get changed port options from the saved point =============
-pkgsys_get_changed_port_oprions ()
+pkgsys_get_changed_port_options ()
 {
 	local saved_log current_log tmp_log
 	saved_log=$1
 	current_log=$2
-	tmp_log=${TMPDIR}/pkgsys_get_changed_port_oprions.log
+	tmp_log=${TMPDIR}/pkgsys_get_changed_port_options.log
 	{
-		grep -vxF -f "$current_log" "$saved_log" || :
-		grep -vxF -f "$saved_log" "$current_log" || :
-	} | cut -w -f 1 | grep -v '^$' | sort -u > $tmp_log
+		env LANG=C grep -vxF -f "$current_log" "$saved_log" || :
+		env LANG=C grep -vxF -f "$saved_log" "$current_log" || :
+	} | cut -w -f 1 | env LANG=C grep -v '^$' | sort -u > $tmp_log
 	if pkgsys_is_dialog4ports_used
 	then
-		grep '_' "$tmp_log" || :
+		env LANG=C grep '_' "$tmp_log" || :
 	else
 		cat "$tmp_log"
 	fi
