@@ -3,7 +3,7 @@
 # portsreinstall library script
 # Overlay onto lib/libcommand_do.sh for portsreinstall-upgrade
 # - "do" command operation -
-# Copyright (C) 2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2018-2022 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -71,8 +71,8 @@ command_do_reinst_ports ()
 		if [ -n "$target_pkg" ]
 		then
 			target_regexp=`str_escape_regexp "$target_pkg"`
-			timestamp_init=`grep -E "^${target_regexp}[[:space:]]" "${DBDIR}/initial_pkgs_snapshot.csv" | cut -f 2`
-			timestamp_fin=`grep -E "^${target_regexp}[[:space:]]" "${ETCDIR}/final_pkgs_snapshot.csv" | cut -f 2`
+			timestamp_init=`env LANG=C grep -E "^${target_regexp}[[:space:]]" "${DBDIR}/initial_pkgs_snapshot.csv" | cut -f 2`
+			timestamp_fin=`env LANG=C grep -E "^${target_regexp}[[:space:]]" "${ETCDIR}/final_pkgs_snapshot.csv" | cut -f 2`
 			if [ -n "$timestamp_init" -a -n "$timestamp_fin" ]
 			then
 				pkgtype_init=`expr "$timestamp_init" : '\([^:]*\):' || :`
@@ -182,8 +182,8 @@ command_do_deinst_remining_olg_pkgs ()
 	{
 		message_section_title "Deinstalling remaining old packages if any..."
 		pkg_get_pkgs_timestamps | \
-			grep -vE "`pkgsys_pkgtools_pkgs_filter_regexp`"> ${TMPDIR}/current_pkgs_snapshot.csv
-		grep -Fxv -f "${ETCDIR}/final_pkgs_snapshot.csv" "${TMPDIR}/current_pkgs_snapshot.csv" | \
+			env LANG=C grep -vE "`pkgsys_pkgtools_pkgs_filter_regexp`"> ${TMPDIR}/current_pkgs_snapshot.csv
+		env LANG=C grep -Fxv -f "${ETCDIR}/final_pkgs_snapshot.csv" "${TMPDIR}/current_pkgs_snapshot.csv" | \
 			cut -f 1 | while read pkg_old
 		do
 			pkg_delete_f "$pkg_old"
